package org.faktorips.tutorial.produktdaten;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.faktorips.runtime.IRuntimeRepository;
import org.faktorips.tutorial.hausrat.hausrat.IHausratProdukt;
import org.faktorips.tutorial.hausrat.hausrat.IHausratProduktGen;
import org.faktorips.tutorial.hausrat.hausrat.IHausratVertrag;
import org.faktorips.tutorial.hausrat.hausrat.IHausratZusatzdeckungstyp;
import org.faktorips.tutorial.modell.base.IDeckung;
import org.faktorips.values.Money;

/**
 * Diese Klasse demonstriert wie die Instanzen der generierten Vertragsklassen mit JAXB nach XML
 * konvertiert werden können (marshalling) und aus XML erzeugt werden können (unmarshalling).
 * 
 * @author Jan Ortmann
 */
public class JAXBDemo {

    public static void main(String[] args) {
        JAXBDemo demo = new JAXBDemo();
        try {
            demo.exec();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private IRuntimeRepository repository;
    private JAXBContext jaxbContext;

    private void exec() throws Exception {
        repository = RuntimeRepositoryFactory.createRuntimeRepository();
        jaxbContext = repository.newJAXBContext();
        IHausratVertrag vertrag = createHausratVertrag();
        vertrag.berechneBeitrag();

        String xml = marshal(vertrag);
        System.out.println(xml);

        IHausratVertrag vertrag2 = unmarshal(xml);
        System.out.print("Bruttobeitrag " + vertrag2.getBruttobeitragZw());
        System.out.println(" fuer den Vertrag auf Basis des Produktes " + vertrag2.getProdukt());
        List<IDeckung> deckungen = vertrag2.getDeckungen();
        for (IDeckung deckung : deckungen) {
            System.out.println(deckung.getDeckungstyp() + ": " + deckung.getJahresbasisbeitrag());
        }
    }

    /*
     * Erzeugt die XML-Repraesentation fuer den uebergebenen HausratVertrag.
     */
    private String marshal(IHausratVertrag vertrag) throws JAXBException {
        Marshaller marshaller = jaxbContext.createMarshaller();
        StringWriter writer = new StringWriter();
        marshaller.marshal(vertrag, writer);
        return writer.toString();
    }

    /*
     * Erzeugt den HausratVertrag aus der gegebenen XML-Repraesentation.
     */
    private IHausratVertrag unmarshal(String xml) throws JAXBException {
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        IHausratVertrag vertrag = (IHausratVertrag)unmarshaller.unmarshal(new StringReader(xml));
        return vertrag;
    }

    private IHausratVertrag createHausratVertrag() {
        IHausratProdukt produkt = (IHausratProdukt)repository.getProductComponent("hausrat.HR-Kompakt 2008-04");
        IHausratVertrag vertrag = produkt.createHausratVertrag();
        vertrag.setPlz("12345");
        vertrag.setVersSumme(Money.euro(50000));
        vertrag.setWirksamAb(new GregorianCalendar());
        vertrag.setWohnflaeche(80);
        vertrag.setZahlweise(1);

        IHausratProduktGen produktGen = (IHausratProduktGen)produkt.getLatestProductComponentGeneration();
        vertrag.newHausratGrunddeckung(produktGen.getHausratGrunddeckungstyp());

        List<? extends IHausratZusatzdeckungstyp> zusatzdeckungsTypen = produktGen.getHausratZusatzdeckungstypen();
        for (IHausratZusatzdeckungstyp zusatzdeckungstyp : zusatzdeckungsTypen) {
            vertrag.newHausratZusatzdeckung(zusatzdeckungstyp);
        }
        return vertrag;
    }

}
