import java.util.GregorianCalendar;

import junit.framework.TestCase;

import org.faktorips.runtime.ClassloaderRuntimeRepository;
import org.faktorips.runtime.IProductComponent;
import org.faktorips.runtime.IProductComponentGeneration;
import org.faktorips.runtime.IRuntimeRepository;
import org.faktorips.tutorial.hausrat.hausrat.IHausratGrunddeckung;
import org.faktorips.tutorial.hausrat.hausrat.IHausratGrunddeckungstyp;
import org.faktorips.tutorial.hausrat.hausrat.IHausratProdukt;
import org.faktorips.tutorial.hausrat.hausrat.IHausratProduktGen;
import org.faktorips.tutorial.hausrat.hausrat.IHausratVertrag;
import org.faktorips.tutorial.hausrat.hausrat.IHausratZusatzdeckung;
import org.faktorips.tutorial.hausrat.hausrat.IHausratZusatzdeckungstyp;
import org.faktorips.values.Money;

public class TutorialTest extends TestCase {

    private IRuntimeRepository repository;
    private IHausratProdukt kompaktProdukt;
    private IHausratProduktGen kompaktGen;

    public void setUp() {
        // wenn das toc-file nicht gelesen werden kann, wird eine runtime exception geworfen.
        repository = ClassloaderRuntimeRepository
                .create("org/faktorips/tutorial/produktdaten/internal/faktorips-repository-toc.xml");

        // Referenz auf das Kompaktprodukt aus dem Repository holen
        IProductComponent pc = repository.getProductComponent("hausrat.HR-Kompakt 2012-03");

        // Juengste Productgeneration holen (wir wissen es gibt nur eine).
        IProductComponentGeneration pcGen = pc.getLatestProductComponentGeneration();

        // Auf die eigenen Modellklassen casten
        kompaktProdukt = (IHausratProdukt) pc;
        kompaktGen = (IHausratProduktGen) pcGen;
    }

    public void testProduktdatenLesen() {
        System.out.println("Produktname: " + kompaktGen.getProduktname());
        System.out.println("Vorschlag Vs pro 1qm: " + kompaktGen.getVorschlagVersSummeProQm());
        System.out.println("Default Zahlweise  : " + kompaktGen.getDefaultValueZahlweise());
        System.out.println("Erlaubte Zahlweisen: " + kompaktGen.getAllowedValuesForZahlweise(null));
        System.out.println("Default Vs: " + kompaktGen.getDefaultValueVersSumme());
        System.out.println("Bereich Vs: " + kompaktGen.getRangeForVersSumme(null));
        System.out.println("Default Wohnflaeche: " + kompaktGen.getDefaultValueWohnflaeche());
        System.out.println("Bereich Wohnflaeche: " + kompaktGen.getRangeForWohnflaeche(null));
    }

    public void testGetTarifzone() {
        // Erzeugen eines hausratvertrags mit der Factorymethode des Produktes
        IHausratVertrag vertrag = kompaktProdukt.createHausratVertrag();

        // Wirksamkeitsdatum des Vertrages setzen, damit die Produktgeneration gefunden wird!
        // Dies muss nach dem Gueltigkeitsbeginn der Generation liegen!
        vertrag.setWirksamAb(new GregorianCalendar(2010, 0, 1));

        vertrag.setPlz("30159");
        assertEquals("III", vertrag.getTarifzone());
    }

    public void testBerechneBeitrag() {
        // Erzeugen eines hausratvertrags mit der Factorymethode des Produktes
        IHausratVertrag vertrag = kompaktProdukt.createHausratVertrag();

        // Wirksamkeitsdatum des Vertrages setzen, damit die Produktgeneration gefunden wird!
        // Dies muss nach dem Gueltigkeitsbeginn der Generation liegen!
        vertrag.setWirksamAb(new GregorianCalendar(2012, 2, 1));

        // Vertragsattribute setzen
        vertrag.setPlz("30159"); // => tarifzone 3
        vertrag.setVersSumme(Money.euro(60000));
        vertrag.setZahlweise(new Integer(2)); // halbjaehrlich

        // Grunddeckungstyp holen, der dem Produkt in der Generation zugeordnet ist.
        IHausratGrunddeckungstyp deckungstyp = kompaktGen.getHausratGrunddeckungstyp();

        // Grunddeckung erzeugen und zum Vertag hinzufuegen
        IHausratGrunddeckung deckung = vertrag.newHausratGrunddeckung(deckungstyp);

        // Beitrag berechnen und Ergebniss pruefen
        vertrag.berechneBeitrag();

        // tarifzone 3 => beitragssatz = 1.21
        // jahresbasisbeitrag = versicherungsumme / 1000 * beitragssatz = 60000 / 1000 * 1,21 = 72,60
        assertEquals(Money.euro(72,60), deckung.getJahresbasisbeitrag());

        // Jahresbasisbeitrag vertrag = Jahresbasisbeitrag deckung
        assertEquals(Money.euro(72, 60), vertrag.getJahresbasisbeitrag());

        // NettobeitragZw = 72,60 / 2 * 1,03 (wg. Ratenzahlungszuschlag von 3%) = 37,389 => 37,39
        assertEquals(Money.euro(37, 39), vertrag.getNettobeitragZw());

        // BruttobeitragZw = 37,39 * Versicherungssteuersatz = 37,39 * 1,19 = 44,49
        assertEquals(Money.euro(44, 49), vertrag.getBruttobeitragZw());
    }

    public void testBerechneJahresbasisbeitragFahrraddiebstahl() {
        // Erzeugen eines hausratvertrags mit der Factorymethode des Produktes
        IHausratVertrag vertrag = kompaktProdukt.createHausratVertrag();

        // Wirksamkeitsdatum des Vertrages setzen, damit die Produktgeneration gefunden wird!
        // Dies muss nach dem Gueltigkeitsbeginn der Generation liegen!
        vertrag.setWirksamAb(new GregorianCalendar(2012, 2, 1));

        // Vertragsattribute setzen
        vertrag.setVersSumme(Money.euro(60000));

        // Zusatzdeckungstyp Fahrraddiebstahl holen
        // Der Einfachheit halber, nehmen wir hier an, der erste ist Fahrraddiebstahl
        IHausratZusatzdeckungstyp deckungstyp = kompaktGen.getHausratZusatzdeckungstyp(0);

        // Zusatzdeckung erzeugen
        IHausratZusatzdeckung deckung = vertrag.newHausratZusatzdeckung(deckungstyp);

        // Jahresbasisbeitrag berechnen und testen
        deckung.berechneJahresbasisbeitrag();

        // Vericherungssumme der Deckung = 1% von 60.000, max 3.000 => 600
        // Beitrag = 5% von 600 = 30 
        assertEquals(Money.euro(30, 0), deckung.getJahresbasisbeitrag());
    }

}
